<?php

declare(strict_types = 1);

namespace Drupal\entity_editor_tabs;

use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\entity_editor_tabs\Plugin\Menu\LocalTask\EetCanonicalLocalTask;
use Drupal\entity_editor_tabs\Plugin\Menu\LocalTask\EetLatestLocalTask;
use Drupal\entity_editor_tabs\Plugin\Menu\LocalTask\EetUpdateLocalTask;
use Drupal\entity_route_context\EntityRouteContextRouteHelperInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Hooks for Entity Editor Tabs.
 */
class EetHooks implements ContainerInjectionInterface {

  /**
   * Provides discovery and retrieval of entity type bundles.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected EntityTypeBundleInfoInterface $bundleInfo;

  /**
   * Manages entity type plugin definitions.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Utility for Entity Editor Tabs.
   *
   * @var \Drupal\entity_editor_tabs\EetUtility
   */
  protected EetUtility $eetUtility;

  /**
   * Route helper.
   *
   * @var \Drupal\entity_route_context\EntityRouteContextRouteHelperInterface
   */
  protected EntityRouteContextRouteHelperInterface $routeHelper;

  /**
   * General service for moderation-related questions about Entity API.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface|null
   */
  protected ?ModerationInformationInterface $moderationInformation;

  /**
   * Class that manages modules in a Drupal installation.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static();
    $instance->bundleInfo = $container->get('entity_type.bundle.info');
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->eetUtility = $container->get('entity_editor_tabs.utility');
    $instance->routeHelper = $container->get('entity_route_context.route_helper');
    $instance->moderationInformation = $container->get('content_moderation.moderation_information', ContainerInterface::NULL_ON_INVALID_REFERENCE);
    $instance->moduleHandler = $container->get('module_handler');
    return $instance;
  }

  /**
   * Implements hook_local_tasks_alter().
   *
   * @see \entity_editor_tabs_local_tasks_alter()
   */
  public function hookLocalTasksAlter(array &$localTasks): void {
    foreach ($this->bundles() as [$entityTypeId, $bundle]) {
      if ($this->moduleHandler->moduleExists('layout_builder') && $this->eetUtility->isLayoutBuilderOverridable($entityTypeId, $bundle)) {
        $routes = $this->routeHelper->getRouteNames($entityTypeId);

        // 'Edit' tab override:
        $editFormRouteName = $routes['edit-form'] ?? NULL;
        if ($editFormRouteName) {
          $editLocalTaskId = $this->getLocalTaskWithRouteName($localTasks, $editFormRouteName);
          if ($editLocalTaskId) {
            $localTasks[$editLocalTaskId]['class'] = EetUpdateLocalTask::class;
          }
        }

        // 'Layout' tab override:
        // From \Drupal\layout_builder\Plugin\SectionStorage\OverridesSectionStorage::buildLocalTasks:
        $overrideTaskId = sprintf('layout_builder_ui:layout_builder.overrides.%s.view', $entityTypeId);
        if (isset($localTasks[$overrideTaskId])) {
          $localTasks[$overrideTaskId]['title'] = \t('Edit content');
        }
      }

      if ($this->moduleHandler->moduleExists('content_moderation') && $this->moderationInformation->shouldModerateEntitiesOfBundle($this->entityTypeManager->getDefinition($entityTypeId), $bundle)) {
        $routes = $this->routeHelper->getRouteNames($entityTypeId);

        // 'View' tab override:
        $canonicalRouteName = $routes['canonical'] ?? NULL;
        if ($canonicalRouteName) {
          $canonicalLocalTaskId = $this->getLocalTaskWithRouteName($localTasks, $canonicalRouteName);
          if ($canonicalLocalTaskId) {
            $localTasks[$canonicalLocalTaskId]['class'] = EetCanonicalLocalTask::class;
          }
        }

        // 'Latest version' tab override:
        $latestVersionRouteName = $routes['latest-version'] ?? NULL;
        if ($latestVersionRouteName) {
          $latestVersionLocalTaskId = $this->getLocalTaskWithRouteName($localTasks, $latestVersionRouteName);
          $localTasks[$latestVersionLocalTaskId]['class'] = EetLatestLocalTask::class;
        }
      }
    }
  }

  /**
   * Generates all bundles in the system as a flat series of arrays.
   */
  protected function bundles() {
    foreach ($this->bundleInfo->getAllBundleInfo() as $entityTypeId => $info) {
      foreach (array_keys($info) as $bundle) {
        yield [$entityTypeId, $bundle];
      }
    }
  }

  /**
   * Finds the first local task representing a route.
   *
   * @param array $localTasks
   *   Local task definitions.
   * @param string $routeName
   *   The route name.
   *
   * @return string|null
   *   The local task ID in $localTasks, or NULL if no task was found.
   */
  protected function getLocalTaskWithRouteName(array $localTasks, string $routeName): ?string {
    foreach ($localTasks as $id => $definition) {
      if (($definition['route_name'] ?? NULL) === $routeName) {
        return $id;
      }
    }
    return NULL;
  }

}
